/* 
 ***********************************************************************
 *
 *                           Copyright ©
 *    Copyright © 2000 Force Computer, Inc., a Solectron company. All rights reserved.
 *    © Digital Equipment Corporation 1998. All rights reserved.
 *
 *    Restricted Rights: Use, duplication, or disclosure by the U.S.
 *    Government is subject to restrictions as set forth in subparagraph
 *    (c) (1) (ii) of DFARS 252.227-7013, or in FAR 52.227-19, or in FAR
 *    52.227-14 Alt. III, as applicable.
 *
 *    This software is proprietary to and embodies the confidential
 *    technology of Force Computers Incoporated and other parties.
 *    Possession, use, or copying of this software and media is authorized
 *    only pursuant to a valid written license from Force or an
 *    authorized sublicensor.
 *
 ***********************************************************************
 *    File Name:    ph_pros.c
 *    Author:       Matthew Schnee
 *    Creation Date:03-mar-1998
 *
 *    Functionality:
 *  	Controls which files get included for re-write of new prosody files
 *
 ***********************************************************************
 *    Revision History:
 *	Rev	Who		Date		Description
 *	---	-----	-----------	--------------------------------------------
 *	001  EAB	03-11-98	Initial creation, ph_drwt0.c was broken into two file
							sets ph_drwt01.c and ph_drwt02.c. The old files contained
							repetive but different variants of the code making editing
							difficult. 
 *  002 CAB		10/18/2000  Changed copyright info
 */



#include "dectalkf.h"



#if (defined(ENGLISH_US) || defined(GERMAN) || defined(SPANISH)) && defined(OLD_INTONATION_AND_TIMING)
#include "ph_drwt01.c"
#else
#include "ph_drwt02.c"
#endif




