/*
 ***********************************************************************
 *                                                                      
 *                           Copyright (c)                              
 *	  Copyright © 2000 Force Computers, Inc., a Solectron Company. All rights reserved.
 *    © Digital Equipment Corporation 1996, 1997. All rights reserved.        
 *                                                                     
 *    Restricted Rights: Use, duplication, or disclosure by the U.S.    
 *    Government is subject to restrictions as set forth in subparagraph
 *    (c) (1) (ii) of DFARS 252.227-7013, or in FAR 52.227-19, or in FAR
 *    52.227-14 Alt. III, as applicable.                                
 *                                                                      
 *    This software is proprietary to and embodies the confidential
 *    technology of Force Computers Incorporated and other parties.
 *    Possession, use, or copying of this software and media is authorized
 *    only pursuant to a valid written license from Force or an
 *    authorized sublicensor.                                            
 *                                                                       
 *********************************************************************** 
 *    File Name:	ls_chari.c
 *    Author:		Matthew Schnee                                         
 *    Creation Date:02/06/96                                                   
 *                                                                             
 *    Functionality:                                                           
 *	  File to include the language dependent character mappings                                                                             
 *                                                                             
 ***********************************************************************       
 *    Revision History:
 *                                                                             
 * Rev	Who		Date			Description                    
 * ---	-----	-----------		---------------------------------------
 * 001	GL		04/21/1997	    BATS#357  add the code for __osf__ build 
 * 002	DR/GL	07/21/1997		UK BUILD: use ENGLISH to replace ENGLISH_US
 * 003  CHJ		07/20/2000		FRENCH BUILD
 * 004 	CAB		10/18/2000		Changed copyright info
 */
#include "ls_def.h"
#if defined (ENGLISH) || defined (GERMAN) || defined (SPANISH) || defined (FRENCH)
#include "l_us_cha.c"
#endif
